/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.finance.portfolio;

import java.math.BigDecimal;
import java.util.List;

import org.ojalgo.ProgrammingError;

/**
 * Shifted return Portfolio
 *
 * @author apete
 */
final class ShiftedPortfolio extends FinancePortfolio {

    private final FinancePortfolio myPortfolio;
    private final double myReturnShift;

    public ShiftedPortfolio(final FinancePortfolio aPortfolio, final Number aReturnShift) {

        super();

        myPortfolio = aPortfolio;
        myReturnShift = aReturnShift.doubleValue();
    }

    @SuppressWarnings("unused")
    private ShiftedPortfolio() {

        this(null, null);

        ProgrammingError.throwForIllegalInvocation();
    }

    @Override
    public double getMeanReturn() {
        return myPortfolio.getMeanReturn() + myReturnShift;
    }

    @Override
    public double getReturnVariance() {
        return myPortfolio.getReturnVariance();
    }

    @Override
    public double getVolatility() {
        return myPortfolio.getVolatility();
    }

    @Override
    public List<BigDecimal> getWeights() {
        return myPortfolio.getWeights();
    }

    @Override
    protected void reset() {
        myPortfolio.reset();
    }

}
