/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.matrix.operation;

import java.math.BigDecimal;

import org.ojalgo.scalar.ComplexNumber;

public final class ApplyLU extends MatrixOperation {

    /**
     * MacPro 20110922: 64
     * iMac 20110922: 128
     */
    public static int THRESHOLD = INT_0064;

    public static void invoke(final BigDecimal[] aData, final int aRowDim, final int aFirstCol, final int aColLimit, final BigDecimal[] aMultipliers, final int iterationPoint) {
        for (int j = aFirstCol; j < aColLimit; j++) {
            CAXPY.invoke(aData, j * aRowDim, aMultipliers, INT_0000, aData[iterationPoint + (j * aRowDim)].negate(), iterationPoint + INT_0001, aRowDim);
        }
    }

    public static void invoke(final ComplexNumber[] aData, final int aRowDim, final int aFirstCol, final int aColLimit, final ComplexNumber[] aMultipliers, final int iterationPoint) {
        for (int j = aFirstCol; j < aColLimit; j++) {
            CAXPY.invoke(aData, j * aRowDim, aMultipliers, INT_0000, aData[iterationPoint + (j * aRowDim)].negate(), iterationPoint + INT_0001, aRowDim);
        }
    }

    public static void invoke(final double[] aData, final int aRowDim, final int aFirstCol, final int aColLimit, final double[] aMultipliers, final int iterationPoint) {
        for (int j = aFirstCol; j < aColLimit; j++) {
            CAXPY.invoke(aData, j * aRowDim, aMultipliers, INT_0000, -aData[iterationPoint + (j * aRowDim)], iterationPoint + INT_0001, aRowDim);
        }
    }

    private ApplyLU() {
        super();
    }

    public int getThreshold() {
        return THRESHOLD;
    }

}
