/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.optimisation;

import java.math.BigDecimal;

/**
 * Constraint
 * 
 * @author apete
 */
public interface Constraint {

    BigDecimal getLowerLimit();

    BigDecimal getUpperLimit();

    /**
     * The Constraint has a lower or an upper limit (possibly both).
     */
    boolean isConstraint();

    /**
     * The Constraint has both a lower limit and an upper limit, and they
     * are equal.
     */
    boolean isEqualityConstraint();

    /**
     * The Constraint has a lower limit, and the upper limit (if it exists)
     * is different.
     */
    boolean isLowerConstraint();

    /**
     * The Constraint has an upper limit, and the lower limit (if it exists)
     * is different.
     */
    boolean isUpperConstraint();

    void setLowerLimit(BigDecimal aValue);

    void setUpperLimit(BigDecimal aValue);

}
