/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.optimisation.linear.network;

import org.ojalgo.ProgrammingError;
import org.ojalgo.constant.BigMath;
import org.ojalgo.optimisation.Variable;

/**
 * Arc
 *
 * @author apete
 */
public final class Arc extends Variable {

    private final Node mySource;
    private final Node myDestination;

    @SuppressWarnings("unused")
    private Arc(final String aName) {

        this(null, null);

        ProgrammingError.throwForIllegalInvocation();
    }

    Arc(final Node aSource, final Node aDestination) {

        super(aSource.getName() + "-" + aDestination.getName());

        mySource = aSource;
        myDestination = aDestination;

        mySource.addOutput(this);
        myDestination.addInput(this);

        this.lower(BigMath.ZERO);
    }

    Arc(final String identifier, final Node aSource, final Node aDestination) {

        super(identifier + ":" + aSource.getName() + "-" + aDestination.getName());

        mySource = aSource;
        myDestination = aDestination;

        mySource.addOutput(this);
        myDestination.addInput(this);

        this.lower(BigMath.ZERO);
    }

    public final Node getDestination() {
        return myDestination;
    }

    public final Node getSource() {
        return mySource;
    }

}
