/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.optimisation.linear.network;

import java.math.BigDecimal;
import java.util.Collection;
import java.util.HashSet;

import org.ojalgo.optimisation.ConstraintsBasedModel;
import org.ojalgo.optimisation.OptimisationSolver;

/**
 * NetworkModel
 *
 * @author apete
 */
public final class NetworkModel extends ConstraintsBasedModel<Node, NetworkModel> {

    private final HashSet<Arc> myVariables = new HashSet<Arc>();

    public NetworkModel(final Node[] someConstraints) {
        super(someConstraints);
    }

    public Arc add(final String identifier, final String aSourceNodeName, final String aDestinationNodeName) {

        Node tmpSource = this.getConstraint(aSourceNodeName);
        if (tmpSource == null) {
            tmpSource = new Node(aSourceNodeName);
            this.addConstraint(tmpSource);
        }

        Node tmpDestination = this.getConstraint(aDestinationNodeName);
        if (tmpDestination == null) {
            tmpDestination = new Node(aDestinationNodeName);
            this.addConstraint(tmpDestination);
        }

        final Arc retVal = new Arc(identifier, tmpSource, tmpDestination);

        myVariables.add(retVal);

        return retVal;
    }

    public OptimisationSolver getDefaultSolver() {
        return null; //TODO new NetworkSolver(this);
    }

    public Node getNode(final String aName) {
        return this.getConstraint(aName);
    }

    public Collection<Node> getNodes() {
        return this.getConstraints();
    }

    public BigDecimal maximise() {
        // TODO Auto-generated method stub
        return null;
    }

    public BigDecimal minimise() {
        // TODO Auto-generated method stub
        return null;
    }

}
