/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.random;

import static org.ojalgo.constant.PrimitiveMath.*;

/**
 * Distribution of length of life when no aging.
 *
 * @author apete
 */
public class Exponential extends AbstractContinuous {

    private static final long serialVersionUID = -720007692511649669L;

    private final double myLambda;

    public Exponential() {
        this(ONE);
    }

    public Exponential(final double aLambda) {

        super();

        myLambda = aLambda;
    }

    public double getDistribution(final double aValue) {
        return ONE - Math.exp(-myLambda * aValue);
    }

    public double getExpected() {
        return ONE / myLambda;
    }

    public double getProbability(final double aValue) {
        return myLambda * Math.exp(-myLambda * aValue);
    }

    public double getQuantile(final double aProbality) {

        this.checkProbabilty(aProbality);

        return Math.log(ONE - aProbality) / -myLambda;
    }

    @Override
    public double getStandardDeviation() {
        return ONE / myLambda;
    }

    @Override
    protected double generate() {
        return -Math.log(this.random().nextDouble()) / myLambda;
    }

}
