/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.random;

import static org.ojalgo.constant.PrimitiveMath.*;

import org.ojalgo.access.Access2D;
import org.ojalgo.array.Array1D;
import org.ojalgo.matrix.decomposition.Cholesky;
import org.ojalgo.matrix.decomposition.CholeskyDecomposition;
import org.ojalgo.matrix.store.MatrixStore;
import org.ojalgo.matrix.store.PhysicalStore;
import org.ojalgo.matrix.store.PrimitiveDenseStore;

public final class MultidimensionalGaussian {

    private final MatrixStore<Double> myCholeskiedCorrelationsMatrix;

    private static final RandomNumber GAUSSIAN = new RandomNumber() {

        public double getExpected() {
            return ZERO;
        }

        @Override
        public double getStandardDeviation() {
            return ONE;
        }

        @Override
        public double getVariance() {
            return ONE;
        }

        @Override
        protected double generate() {
            return this.random().nextGaussian();
        }
    };

    public MultidimensionalGaussian(final Access2D<?> aCorrelationsMatrix) {

        super();

        final Cholesky<Double> tmpCholesky = CholeskyDecomposition.makePrimitive();

        tmpCholesky.compute(PrimitiveDenseStore.FACTORY.copy(aCorrelationsMatrix)); // Shouldn't have to copy here...

        myCholeskiedCorrelationsMatrix = tmpCholesky.getL();

        tmpCholesky.reset();
    }

    @SuppressWarnings("unused")
    private MultidimensionalGaussian() {
        this(null);
    }

    public Array1D<Double> generate() {

        final int tmpDim = myCholeskiedCorrelationsMatrix.getMinDim();

        final PhysicalStore<Double> tmpUncorrelated = PrimitiveDenseStore.FACTORY.makeZero(tmpDim, 1);
        for (int i = 0; i < tmpDim; i++) {
            tmpUncorrelated.set(i, 0, GAUSSIAN.generate());
        }

        return ((PrimitiveDenseStore) tmpUncorrelated.multiplyLeft(myCholeskiedCorrelationsMatrix)).asArray2D().asArray1D();
    }

}
