/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package org.ojalgo.random.process;

import org.ojalgo.random.Distribution;
import org.ojalgo.random.SampleSet;

/**
 * @author apete
 */
public interface RandomProcess<D extends Distribution> {

    /**
     * Calling this method repeatedly gives the same ressult.
     * 
     * @param aStepSize How far into the future?
     * @return The distribution for the process value at that future time.
     */
    D getDistribution(double aStepSize);

    /**
     * @return An array of sample sets. The array has aNumberOfSteps
     * elements, and each sample set has aNumberOfRealisations samples.
     */
    SampleSet[] simulate(final int aNumberOfRealisations, final int aNumberOfSteps, final double aStepSize);

    /**
     * Calling this method repeatedly gives different, random, ressults.
     * Further, calling this method should update the process state so
     * that subsequent calls to getDistribution would be different.
     * 
     * @param aStepSize How far into the future?
     * @return The new (randomly generated) process value at that future time.
     */
    double step(double aStepSize);

}
