/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Regardless of how you got access to this file;
 * the code is not yours, and you're not allowed to use it in any way.
 */
package org.ojalgo.type.structure;

import java.math.BigDecimal;
import java.util.List;

import org.ojalgo.type.attribute.AggregatedAmountAttribute;
import org.ojalgo.type.attribute.AggregatedQuantityAttribute;
import org.ojalgo.type.attribute.NameAttribute;

/**
 * A ContentStructure represents some sort of content (that has a price).
 * It has references to its Item (what is held) and Container (where is
 * it held). Think of a ContentStructure.Container as a Portfolio and a
 * ContentStructure.Item as an instrument/asset, but they really could
 * be anything.
 *
 * @author apete
 */
public interface ContentStructure<C extends ContentStructure.Container, I extends ContentStructure.Item> extends QuantityPriceAmountStructure {

    public interface Container extends NameAttribute, AggregatedAmountAttribute {

        List<? extends ContentStructure<?, ?>> getAggregationContents();

    }

    public interface Item extends NameAttribute, AggregatedQuantityAttribute, AggregatedAmountAttribute {

        List<? extends ContentStructure<?, ?>> getAggregationContents();

    }

    C getContentContainer();

    I getContentItem();

    /**
     * @return Typically amount / container.aggregatedAmount
     */
    BigDecimal getWeight();

}
