/*
 * Copyright 2003-2011 Optimatika (www.optimatika.se)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package se.optimatika.jfree.data;

import java.util.Map;

import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.TableXYDataset;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.ojalgo.chart.BasicChart;
import org.ojalgo.series.NumberSeries;

/**
 * XYSeriesData
 * 
 * @author apete
 */
public class NumberSeriesCollection extends AbstractSeriesData<Number, Number> {

    private final XYSeriesCollection myCollection = new XYSeriesCollection();

    public NumberSeriesCollection() {
        super();
    }

    public void addSeries(final NumberSeries<?> aSeries) {

        final XYSeries tmpSeries = new XYSeries(aSeries.getName(), true, false);

        Number tmpKey;
        Number tmpValue;
        for (final Map.Entry<? extends Number, ? extends Number> tmpEntry : aSeries.entrySet()) {
            tmpKey = tmpEntry.getKey();
            tmpValue = tmpEntry.getValue();
            tmpSeries.add(tmpKey, tmpValue);
        }

        myCollection.addSeries(tmpSeries);

        this.setSeriesPaint(aSeries.getName(), aSeries.getColour());
    }

    @SuppressWarnings("unchecked")
    public <T> BasicChart<T> build() {
        //return (BasicChart<T>) this.buildScatterPlot();
        return (BasicChart<T>) this.buildStackedXYAreaChart();
    }

    @Override
    protected IntervalXYDataset getIntervalXYData() {
        return myCollection;
    }

    @Override
    protected TableXYDataset getTableXYData() {

        final DefaultTableXYDataset retVal = new DefaultTableXYDataset();

        for (final Object tmpXYSeries : myCollection.getSeries()) {
            retVal.addSeries((XYSeries) tmpXYSeries);
        }

        return retVal;
    }

    @Override
    protected XYDataset getXYData() {
        return myCollection;
    }

    @Override
    protected boolean isDomainTime() {
        return false;
    }
}
